;;-------------------------------------------------------------------------;;    This file is part of Catpang.
;;    Copyright (C) 2018-2019 Débora Galdeano González / Mª Inés Anton Castelló	
;;    Catpang is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Catpang is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Catpang.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------


.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "nivel.h.s"

.globl _bala_sp

;; DefineBala _name, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd, _vx, _vy
DefineBala bala, #FALSE, #xCentro, #yPrincipio, #0x01, #0x04, #_bala_sp, #0, #yPrincipio, #bala_update, #0, #0x04


;;======================================
;;        Punteros a bala
;;======================================
bala_getPtrIX:
	ld 		ix, #bala
ret


;;======================================
;;	Cambiar posicion X de la bala
;;	Registros destruidos: AF
;;  Entrada IX -> Puntero a bala
;;======================================
cambiar_x:
	ex 		af, af'					;; Pos. x de personaje
	ld 		de_x(ix), a 			;; Le metemos la x del personaje
ret


;;======================================
;;	Reiniciar valores de una bala
;;  Entrada IX -> Puntero a bala
;;======================================
reiniciar_bala:
	;; Borramos la antigua posicion
	call ent_restart

	;; Asignamos la nueva posicion
	call recargar_bala
ret


recargar_bala:
	;; La antigua posicion la borra el newScene
	;; cuando llamamos al pasar fuera limite

	;; Asignamos la nueva posicion
	ld 		e_activada(ix), #FALSE
	call 	cambiar_x
	ld 		de_x(ix), #xCentro
	ld 		de_y(ix), #yPrincipio
ret


;;======================================
;;	Actualizar
;;	Registros destruidos: A
;;  Entrada IX -> Puntero a bala,  A' -> pos.X personaje
;;======================================
bala_update:
	
	;; Guardar la posicion antigua de X e Y
	call 	intercambiarPosiciones

	;; Comprobamos si esta activa la bala
	ld 		a, e_activada(ix)			;; e_activada es FALSE, 0
	cp 		#FALSE						;; 0-0 = 0, Z=1
	jr 		z, bala_desactivada 		;; Si Z=1, salta a desactivada

		;; Bala activada
		call 	bala_move
		ret

	bala_desactivada:
		call 	bala_teclado
ret


;;======================================
;;	Leer teclado
;;	Registros destruidos: HL, A, L
;;  Entrada IX -> Puntero a bala
;;======================================
bala_teclado:
	call 	cpct_scanKeyboard_asm		;; Escanear todas las teclas

	ld 		hl, #Key_Space 				;; HL = Key_Space Keycode
	call 	cpct_isKeyPressed_asm		;; Comprueba si Key_Space esta pulsada
	jr		z, space_no_pulsada			;; si Z=1, salta a space_no_pulsada

		;; Space pulsada, Z=0
		ld 		e_activada(ix), #TRUE
		call 	cambiar_x

	space_no_pulsada:
ret


;;======================================
;;	Mover una bala
;;	Registros destruidos: BC, A
;;  Entrada IX -> Puntero a bala
;;======================================
bala_move:
	ld 		a, de_y(ix)
	cp 		#limiteArb				;; 0-0 = 0, Z=1
	jr 		z, fuera_limite			;; Si Z=1, salta a fuera_limite

		;; Dentro del limite
		ld 		a, de_y(ix)		;; Mete en A la pos. Y
		sub 	e_vy(ix)		;; Le resta la velocidad, Y = Y-VY
		ld 		de_y(ix), a 	;; Mete en la pos. Y la nueva pos.
		ret

	;; Colisiona con el techo
	fuera_limite:
		call recargar_bala
ret


bala_clear:
	call 	bala_getPtrIX
	call 	ent_clear
ret