;;-------------------------------------------------------------------------;;    This file is part of Catpang.
;;    Copyright (C) 2018-2019 Débora Galdeano González / Mª Inés Anton Castelló	
;;    Catpang is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Catpang is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Catpang.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------


.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "nivel.h.s"

;; Datos iniciales bolas
x_ini = #8				;;30
y_ini = #150			;;199

x_iniM = #60
y_iniM = #150

tam_ini = #3			;; 3 tamanyos de bolas
grande = #3
mediana = #2
peque = #1

;; Tamanyos: Width : in  mode 0	1 byte = 2 pixels

;; ------------- 26x42
w_26 = #0x0D 		;; 26/2=13 -> D
h_26 = #0x2A

;; ------------- 16x26
w_16 = #0x08		;; 16/2=8 -> 8
h_16 = #0x1A

;; ------------- 8x14
w_8 = #0x04
h_8 = #0x1A

w_ini = #w_26
h_ini = #h_26


;; DefineNBalls _name, _nBalls
bola_grande_vector:
DefineNBalls bola_grande_vector, k_max_num_balls
;; DefineBall 	_name, _activa, _x,  	_y, 	_w, 	_h, 	_sprite, 	_xAntigua, _yAntigua, _upd, 	_vx, _vy, _fsalt_ini, _tam
DefineBall bola_grande, #TRUE, #x_ini, #y_ini, #w_ini, #h_ini, #_b_grande_sp, #x_ini, #y_ini, bola_update, #0x01, #0x03, #0x0F, #tam_ini
;; Tamanyo: #0x04, #0x0F

m_num_ball:		.db 00						;; memoria inicial de cantidad de bolas
m_next_ball: 	.dw bola_grande_vector		;; puntero hacia la primera bola

;;======================================
;;        Punteros a bola_grande
;;======================================
bola_grande_getPtrIX:
	ld 		ix, #bola_grande
ret
bola_grande_getPtrIY:
	ld 		iy, #bola_grande
ret
bola_grande_getPtrHL:
	ld 		hl, #bola_grande
ret
bola_grande_vector_getPtrIX:
	ld 		ix, #bola_grande_vector
ret
bola_grande_vector_getPtrIY:
	ld 		iy, #bola_grande_vector
ret
bola_grande_vector_getPtrHL:
	ld 		hl, #bola_grande_vector
ret
getM_bolas:
	ld 		a, (m_num_ball)
ret

;;======================================
;;	Registra una nueva bola
;;	Registros destruidos: AF, DE, HL
;;  Salida HL -> Puntero a bola
;;======================================

bola_new:
	ld 		a, (m_num_ball)
	inc 	a 							;; Incrementa la cantidad de bolas activas
	ld 		(m_num_ball), a				;; se guarda la nueva cantidad de bolas activas en memoria

	ld 		hl, (m_next_ball)
	ld 		bc, #k_balls_size
	add		hl, bc 						;; hl apunta a la siguiente bola del array
	ld 		(m_next_ball), hl 			;; Se guarda en m_next_ball la posición de memoria de la nueva siguiente bola del array
	;;ld 		bc, #-k_balls_size
	;;add 	hl, bc
	or 		a 							;; Se elimina el acarreo para no tener problemas con SBC
	sbc 	hl, bc 						;; hl vuelve a apuntar a la vola actual y no a la siguiente

	ret

;;======================================
;;	Elimina las bolas
;;	Registros destruidos: AF, DE, HL
;;  Salida HL -> Puntero a bola
;;======================================

bola_erase:
	ld 		a, (m_num_ball)
	
	loop_erase_ball:
	ld 		hl, (m_next_ball)
	ld 		bc, #k_balls_size
	push 	af
	or 		a 							;; Se elimina el acarreo para no tener problemas con SBC
	sbc 	hl, bc 						;; hl vuelve a apuntar a la vola actual y no a la siguiente
	pop 	af					;; hl apunta a la siguiente bola del array
	ld 		(m_next_ball), hl 			;; Se guarda en m_next_ball la posición de memoria de la nueva siguiente bola del array
	
	dec  	a
	cp 		#0
	jr 	nz, loop_erase_ball
	ld 		(m_num_ball), a

	ret

;;======================================
;;	Copia los valores de una nueva bola en otra
;;	Registros destruidos:
;;  Entrada
;;		 HL -> Bola origen
;;		 DE -> Bola destion
;;======================================

bola_copy:
	ld 		bc, #k_balls_size
	ldir
	ret

;;======================================
;;	Actualizar
;;	Registros destruidos: HL
;;  Entrada IX -> Puntero a bola
;;======================================
bola_update:
	call 	cpct_scanKeyboard_asm		;; Escanear todas las teclas

	ld 		hl, #Key_N 				;; HL = Key_Space Keycode
	call 	cpct_isKeyPressed_asm		;; Comprueba si Key_Space esta pulsada
	jr 		z, no_nueva_bola

	call	bola_new		;; vector + 1 y hl contiene la nueva bola
	ex 		de, hl
	ld 		hl, #bola_grande
	call 	bola_copy

	no_nueva_bola:
	ld 		a, (m_num_ball) 			;; Cargamos el total de bolas
	call 	bola_grande_vector_getPtrIY ;; Cargamos la posición de inicio del vector de bolas
	ld 		hl, #bola_saltar
	call 	ent_doForAll_Y

ret


;;======================================
;;	Rebote horizontal de las bolas
;;	Registros destruidos: A
;;  Entrada IX -> Puntero a bola
;;======================================
bola_rebote_horizontal:
	
	ld 		a, de_x(iy)		;; Mete en A la posicion X
	sub 		#limiteIzq		;; Compara la posicion de la bola con la de la pared izquierda

	jr		z,	rebotaIzq	;; Si coniciden salta a rebotaIzq

	add 	de_w(iy)			;; Suma el ancho de la bola a A

	sub		#limiteDer		;; Compara la posicion de la bola con la de la pared derecha
	jr		nz,	termina		;; Si no coinciden salta al final de la funcion

	ld 		e_vx(iy), #-1	;; En caso de rebote derecho, se invierte el valor de la velocidad

	jp 		termina

	rebotaIzq:

	ld 		e_vx(iy), #1	;; En caso de rebote iaquierdo, se invierte el valor de la velocidad

	termina:
ret


;======================================
;;	Rebote vertical de las bolas
;;	Registros destruidos: A
;;  Entrada IX -> Puntero a bola
;;======================================
bola_rebote_vertical:

	ld 		a, de_y(iy)					;; Mete en A la pos. Y.
	add 	de_h(iy)					;; Suma a A la altura del objeto
	sub		#limiteAbj					;; Compara la posición de la bola con la del suelo
	jr		z,	rebota				;; En caso de no coincidir, termina

	ld 		a, de_y(iy)					;; Mete en A la pos. Y.
	sub 	#limiteAbj					;; Compara la posición de la bola con la del suelo
	
	jr		nz,	continuar				;; En caso de no coincidir, termina

	rebota:
	ld 		a, e_fsalt_ini(iy)			;; En caso de coinicidir, reinicia la velocidad de salto
	ld 		e_vy(iy), a 				;; Mete el valor de A a la velocidad en Y

	continuar:
ret


;======================================
;;	Programa para los saltos de los enemigos
;;	Registros destruidos: A
;;  Entrada IX -> Puntero a bola
;;======================================
bola_saltar:

	;; Guardar la posicion antigua de X e Y
	ld 		a, de_x(iy)
	ld 		xAntigua(iy), a
	ld 		a, de_y(iy)
	ld 		yAntigua(iy), a

	call 	colision_bala_bola
	cp 		#TRUE						;; A-1 = 0, Z=1
	jr 		z, igual					;; Si Z=1, salta a igual porque colisiona

	call 	colision_bola_jugador
	cp 		#TRUE						;; A-1 = 0, Z=1
	jr 		z, igual_jugador			;; Si Z=1, salta a igual_jugador porque colisiona	

	call 	bola_rebote_vertical
	call 	bola_rebote_horizontal

	ld 		a, de_y(iy)				;; Mete en A la pos. Y
	sub 	e_vy(iy)				;; Le suma la velocidad de salto
	ld 		de_y(iy), a 			;; Mete en la pos. Y la nueva pos.

	call 	bola_move

	ld 		a, e_vy(iy)				;; Mete en A la fuerza de salto
	dec 	a						;; Le suma la velocidad
	ld 		e_vy(iy), a 			;; Mete en e_vy la nueva velocidad de salto

	ld 		a, #FALSE	;; Utiliza IY, por eso indica IX-FALSE
	call 	ent_draw

	ret

	;; Colisiona con bala
	igual:
	;; Borramos todo (menos hud) de los 2 buffers
	call 	clearAhora
	call 	ren_switchBuffers
	call 	clearAhora
	call 	ren_switchBuffers

	;; Asignamos el nuevo Sprite a bola
	call 	cambiar_sprite

	;; Reiniciamos la bala
	call 	bala_getPtrIX
	call 	recargar_bala
	ret

	;; Colisiona con jugador
	igual_jugador:
	call 	jugador_vida_reduce

ret


;;======================================
;;	Mover una Bola
;;	Registros destruidos: A
;;  Entrada IX -> Puntero a bola
;;======================================
bola_move:
	ld 		a, de_x(iy)		;; Mete en A la pos. X
	add 	e_vx(iy)		;; Le suma la velocidad
	ld 		de_x(iy), a 	;; Mete en la pos. X la nueva pos.
ret

;;======================================
;;	Comprobar si colisiona la bala con la bola
;;	Registros destruidos: A, B, C
;;  Entrada IX -> bala, IY -> bola
;;======================================
colision_bala_bola:
	
	;; Primero mover y luego comprobar posiciones resultantes
	call 	bala_getPtrIX
	ld 		a, e_activada(ix)
	cp 		#TRUE						;; A-1 = 0, Z=1
	jr		nz,	no_colision 			;; Si Z=0, salta a no colision

	;; If (bala_x + bala_w <= bola_x) no_colisiona
	;; bala_x + bala_w - bola_x <= 0

	ld 		a, de_x(ix)
	add 	de_w(ix)
	sub 	de_x(iy)					;; Si la resta A-HL=0, flag Z activado
									;; si A-HL>0 (positivo), flag S=0
									;; si A-HL<0 (negativo), flag S=1
	jr		z, no_colision 	;; Si <=0
	jp 		m, no_colision 	;; entonces no colisiona

	;; Colision
	;; If ((bala_x + bala_w) <= (bola_x + bola_w)) colisiona
	;; (bala_x + bala_w) - (bola_x + bola_w) <= 0

		ld 		a, de_x(ix)
		add 	de_w(ix)
		ld 		c, a 			;; (bala_x + bala_w) -> C

		ld 		a, 	de_x(iy)
		add 	de_w(iy)			
		ld 		b, a 			;; (bola_x + bola_w) -> B
		ld 		a, c 			;; C (bala) -> A
		sub 	b

		jr		z, colision 	;; Si Z=1 salta a colisiona
		jp 		m, colision

			jp no_colision

	colision:
	;; Colision
	;; If (bola_y + bola_h <= bala_y) no_colisiona
	;; bola_y + bola_h - bala_y <= 0
		ld 		a, de_y(iy)
		add 	de_h(iy)						;; e_y_bola + e_h_bola
		sub 	de_y(ix)						;; - e_y_bala

		jr		z, no_colision 	;; Si <=0
		jp 		m, no_colision 	;; entonces no colisiona

		;; Colision
		;; If (bala_y + bala_h <= bola_y ) no_colisiona
		;; bala_y + bala_h - bola_y <= 0
		ld 		a, de_y(ix)
		add 	de_h(ix)						;; bala_y + bala_h
		sub 	de_y(iy)						;; - bola_y

		jr		z, no_colision 	;; Si <=0
		jp 		m, no_colision 	;; entonces no colisiona

		;; Colision
		ld 		a, #TRUE
		ret
	no_colision:
		ld 		a, #FALSE
ret

;;======================================
;;	Colision de la bola y jugador
;;	Registros destruidos: A, B, C, HL
;;  Entrada IX -> entidad, hl -> bola
;;======================================

colision_bola_jugador:

call 	personaje_getPtrIX		;; Empieza en e_x

ld 		a, e_invulnerable(ix)
cp 		#TRUE
jr 		z, no_colisiona_invuln

ld 		a, e_activada(iy)
cp 		#TRUE
jr 		nz, no_colisiona

;;	condicion1:
;; if (jugador_x + jugador_w <= bola_x) then no_colisiona
;; (jugador_x + jugador_w - bola_x <= 0)

ld 		a, de_x(ix)
add		de_w(ix)
sub 	de_x(iy)

jr 		z, no_colisiona		;;si A-HL =0, no colisiona
jp		m, no_colisiona		;;si A-HL <0, no colisiona

;; condicion2:
;; if !condicion1 && (jugador_x + jugador_w <= bola_x + bola_w) then colisiona
;; (jugador_x + jugador_w - bola_x <= 0)

ld 		a, de_x(ix)
add		de_w(ix)
ld 		c, a

ld 		a, de_x(iy)				
add		de_w(iy)				

ld 		b, a 				;; b = de_x(bola) + de_x(bola)
ld      a, c 				;; a = de_x(jugador) + de_x(jugador)
sub 	b

jr 		z, colisiona		;;si A-B =0, colisiona
jp		m, colisiona		;;si A-B <0, colisiona	
	jp	no_colisiona

colisiona:
;; if condicion1 && condicion2 && (bola_y + bola_h <= jugador_y) no_colisiona
;; bola_y + bola_h - jugador_y <= 0

	ld 		a, de_y(iy)
	add 	de_h(iy)					;; de_y_bola + de_h_bola
	sub 	de_y(ix)						;; - de_y_bala

	jr		z, no_colisiona 	;; Si <=0
	jp 		m, no_colisiona 	;; entonces no colisiona

;; Colisiona, ahora hay que ver si el personaje es invulnerable

	ld 		e_invulnerable(ix), #TRUE
	ld		e_parpadeo(ix), #TRUE

	;; Colision
	ld 		a, #TRUE

	ret

;; Aunque la bola deba colisionar, es invulnerable, asi que no hay danyo
	
	no_colisiona_invuln:
	call 	aumenta_contador_invuln		;; Aumenta el contador de invulnerabilidad
	cp 		#finInvuln
	jr 		nz, sigue_invuln			;; Z=0, continua invulnerable

	call reset_invulnerable				;; Termina la invulnerabilidad y salga a no_colisiona

	jp 		no_colisiona

	;; Continua invulnerable
	sigue_invuln:
	call 	aumenta_contador_parpadeo	;; Aumenta el contador del parpadeo
	cp 		#finParp
	jr 		nz, no_colisiona 			;; Z=0, salta a no_colisiona por lo que no cambia su visibilidad
	
	ld 		a, e_parpadeo(ix)			;; Cambia el estado de visibilidad, se comprueba que cambio se debe hacer
	cp 		#TRUE
	jr 		nz, invisible				;; Z=0, salta a invisible

	ld 		e_parpadeo(ix), #FALSE		;; el jugador se vuelve visible

	jp 		reinicia_contador

	invisible:

	ld 		e_parpadeo(ix), #TRUE		;; el jugador se vuelve invisible

	reinicia_contador:					;; tras el cambio de visibilidad se reinicia el contador del parpadeo
	ld 		e_cont_parp(ix), #iniContGATO

	no_colisiona:
	ld 		a, #FALSE

ret	


;;======================================
;;	Funciones para cambiar el sprite y tamnyo de la bola
;;  Entrada IY -> Puntero a bola
;;======================================
asignar_sprite_26:
	ld 		hl, #_b_grande_sp
	ld 		de_sprite_l(iy), l
	ld 		de_sprite_h(iy), h
	ld 		de_w(iy), #w_26
	ld 		de_h(iy), #h_26
ret
asignar_sprite_16:
	ld 		hl, #_b_mediana_sp
	ld 		de_sprite_l(iy), l
	ld 		de_sprite_h(iy), h
	ld 		de_w(iy), #w_16
	ld 		de_h(iy), #h_16
ret
asignar_sprite_8:
	ld 		hl, #_b_peque_sp
	ld 		de_sprite_l(iy), l
	ld 		de_sprite_h(iy), h
	ld 		de_w(iy), #w_8
	ld 		de_h(iy), #h_8
ret

;;======================================
;;	Cambiar sprite de la BOLA cuando collisione con BALA
;;	Registros destruidos: A
;;  Entrada IX -> Puntero a bola
;;======================================
cambiar_sprite:
	;call 	bola_grande_getPtrIX
	;; La bala ha colisionado y le bajamos 1 tamanyo, 		3-grande 	2-mediana 	1-pequenya

	;; Comprobamos si hay que desactivarla, (tam == 0)
	dec 	e_tam(iy)
	ld 		a, e_tam(iy)
	cp 		#0						;; A-0 = 0, Z=1
	jr 		z, desactivar			;; Si Z=1, salta a desactivar

	;; Comprobamos si es pequenya
	cp 		#peque					;; A-1 = 0, Z=1
	jr 		z, salta_peque			;; Si Z=1, salta a salta_peque

		;; Comprobamos si es mediana
		cp 		#mediana			;; A-2 = 0, Z=1
		jr 		nz, saltar			;; Si Z=1, salta a saltar

		call 	asignar_sprite_16
		ret

	salta_peque:
		call 	asignar_sprite_8
		ret

	desactivar:
	;; Desactivamos
	ld 		e_activada(iy), #FALSE
	call 	nivel_getPtrIX
	ld 		a, e_nivelNumBalls(ix)
	dec 	a
	ld 		e_nivelNumBalls(ix), a
	ret

	saltar:
ret

bola_clear:
	ld 		a, (m_num_ball) 			;; Cargamos el total de bolas
	call 	bola_grande_vector_getPtrIX ;; Cargamos la posición de inicio del vector de bolas
	ld 		hl, #ent_clear
	call 	ent_doForAll_X
ret
;;======================================
;;	Reiniciar valores de un bola
;;  Entrada IX -> Puntero a bola
;;======================================
reiniciar_bola:
	call ent_restart
	;; TODO: ajustar valores personalizados para cada bola
	
	ld 		de_x(ix), #x_ini
	ld 		de_y(ix), #y_ini
	ld 		e_tam(ix), #tam_ini
	ld 		hl, #_b_grande_sp
	ld 		de_sprite_l(ix), l
	ld 		de_sprite_h(ix), h
	ld 		de_w(ix), #w_26
	ld 		de_h(ix), #h_26
	ld 		e_activada(ix), #TRUE
ret
;;======================================
;;	Reiniciar valores de las bolas
;;  Entrada IX -> Vector de bolas
;;======================================
reiniciar_bolas:
	ld 		a, (m_num_ball) 			;; Cargamos el total de bolas
	call 	bola_grande_vector_getPtrIX ;; Cargamos la posición de inicio del vector de bolas
	ld 		hl, #reiniciar_bola
	call 	ent_doForAll_X