;;-------------------------------------------------------------------------;;    This file is part of Catpang.
;;    Copyright (C) 2018-2019 Débora Galdeano González / Mª Inés Anton Castelló	
;;    Catpang is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Catpang is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Catpang.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------


;; Macro global para dibujar entidades
.macro DefineEntityDibujable _name, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd
_name:
	.db		_activa					;; Estado, 0 = FALSE, 1 = TRUE
	.db 	_x, _y 					;; Posicion X, Y
	.db 	_w, _h					;; Tamaño W, H
	.dw 	_sprite					;; Puntero al sprite
	.db 	_xAntigua, _yAntigua	;; Pos. X e Y antiguas -> dibujar el buffer
	.dw		_upd
	_name'_SIZE = . - _name			;; Size de la entidad
.endm
e_activada 	= 0									;; Estado
de_x 		= 1			de_y 		= 2			;; Posiciones
de_w 		= 3			de_h 		= 4			;; Ancho y Alto
de_sprite_l = 5 		de_sprite_h = 6			;; Puntero al sprite, byte bajo y alto
xAntigua 	= 7			yAntigua 	= 8
e_upd_l 	= 9 		e_upd_h 	= 10 		;; Posicion del update, byte bajo y alto

e_de_size 	= 11 		;; Pendiente de automatizar


;; Macro para definir el personaje
.macro DefineHero _name, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd, _vx, _vidas, _invulnerable, _cont_invul, _parpadeo, _cont_parp
_name:
	DefineEntityDibujable _name'_dw, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd
	.db 	_vx					;; VX
	.db 	_vidas
	.db 	_invulnerable
	.db 	_cont_invul
	.db 	_parpadeo
	.db 	_cont_parp
.endm


;; Macro para definir balas
.macro DefineBala _name, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd, _vx, _vy
_name:
	DefineEntityDibujable _name'_dw, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd
	.db 	_vx, _vy			;; VX, VY
	_name'_SIZE = . - _name
.endm


;; Macro para definir bolas
.macro DefineBall _name, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd, _vx, _vy, _fsalt_ini, _tam
_name:
	DefineEntityDibujable _name'_dw, _activa, _x, _y, _w, _h, _sprite, _xAntigua, _yAntigua, _upd
	.db 	_vx, _vy			;; VX, VY
	.db		_fsalt_ini			;; Fuerza del salto de la entidad
	.db 	_tam				;; Tamanyos de bolas disponibles
	_name'_SIZE = . - _name		;; Size de la bola, bola_SIZE
.endm

;; Macros para arrays
;; Arrays de bolas

.macro DefineBallDefault _name, _suf
	DefineBall _name'_suf, 0xAA, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xFF
.endm
.macro DefineNBalls _name, _nBalls
	_c = 0
	.rept _nBalls

		;;Variables que perimiten generar un aposición inicial distinta por cada bola
		DefineBallDefault _name, \_c
		_c = _c + 1
	.endm
.endm

;; Hero, Ball, Shot
e_de 			= 0					;; Entidad dibujable
e_vx 			= 0 + e_de_size

;; Hero
e_vida			= 1 + e_de_size		;; Vida del Hero
e_invulnerable	= 2 + e_de_size		;; Activacion de invulnerabilidad
e_cont_invuln	= 3 + e_de_size		;; Contador de invulnerabilidad tras ataque
e_parpadeo 		= 4 + e_de_size
e_cont_parp		= 5 + e_de_size

;; Ball, Shot
e_vy 			= 1 + e_de_size		;; Velocidades

;; Ball
e_fsalt_ini		= 2 + e_de_size		;; Altura para restaurar el rebote
e_tam 			= 3 + e_de_size

k_max_num_balls = 6					;; Cantidad de bolas (dependera del nivel)
k_balls_size 	= 4 + e_de_size		;; Tamaño de la entidad ball (coincide con su ultimo parametro)


.globl bola_grande
;.globl bola_mediana
.globl bola_grande_vector
.globl m_num_ball
.globl m_next_ball
.globl getM_bolas
.globl bola_new
.globl bola_copy
.globl bola_erase


;;======================================
;;        Sprites
;;======================================
.globl _b_grande_sp
.globl _b_mediana_sp
.globl _b_peque_sp


;;======================================
;;        Punteros a entidades
;;======================================
.globl personaje_getPtrIX
.globl bala_getPtrIX

.globl bola_grande_getPtrIX
.globl bola_grande_getPtrIY
.globl bola_grande_getPtrHL
.globl bola_grande_vector_getPtrIX
.globl bola_grande_vector_getPtrIY
.globl bola_grande_vector_getPtrHL


;;======================================
;;        Vida del jugador
;;======================================
.globl hud_draw
.globl hud_clear
.globl vida_delete
.globl jugador_vida_reduce


;;======================================
;;		  Reiniciar entidades
;; 	Al pulsar R desde MAIN, se llama a estas
;;======================================
.globl reiniciar_partida
.globl reiniciar_personaje
.globl reiniciar_bolas
.globl reiniciar_bala
.globl reiniciar_vidas
.globl ent_restart


;;======================================
;;        Funciones globales
;;======================================
.globl ren_InitBuffer
.globl ren_newScene
.globl ren_switchBuffers

.globl ent_doForAll_X
.globl ent_doForAll_Y
.globl ent_draw

.globl ent_update

.globl ent_clear
.globl clearAhora
.globl personaje_clear
.globl bala_clear
.globl bola_clear

.globl intercambiarPosiciones
.globl recargar_bala
.globl colision_bala_bola
.globl colision_bola_jugador
.globl aumenta_contador_invuln
.globl reset_invulnerable
.globl aumenta_contador_parpadeo

.globl bolas_pruebas

.globl menu_draw
.globl menu_clear

.globl win_draw
.globl win_clear

.globl lose_draw
.globl lose_clear

.globl easter_egg
.globl trans_win
.globl trans_lose