;;-------------------------------------------------------------------------;;    This file is part of Catpang.
;;    Copyright (C) 2018-2019 Débora Galdeano González / Mª Inés Anton Castelló	
;;    Catpang is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Catpang is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Catpang.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------


.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "nivel.h.s"


m_front_buffer: .db 0xC0
m_back_buffer:  .db 0x80


;;======================================
;; Limpia y prepara el buffer 8000
;;======================================
ren_InitBuffer:
	ld 		hl, #0x8000
	ld 		(hl), #0
	ld 		de, #0x8000 + 1
	ld 		bc, #0x4000 - 1
	ldir
ret


;;======================================
;; Limpia y prepara el siguiente buffer
;;======================================
ren_newScene:
	call 	ren_switchBuffers
	call 	ren_clearBackBuffer
ret


;;======================================
;; Limpia el buffer
;;======================================
ren_clearBackBuffer:
	
	call personaje_clear
	call bala_clear
	call bola_clear
	call hud_clear
ret


;;=======================================================
;; Para limpiar los buffer despues de colision bola-bala
;;=======================================================
clearAhora:
	call 	personaje_getPtrIX
	call 	ent_clearAhoraIX

	call 	bala_getPtrIX
	call 	ent_clearAhoraIX
	
	call 	bola_clear
ret


;;======================================
;; Intercambia los buffers
;; Registros destruidos: A, B, L
;;======================================
ren_switchBuffers:
	ld 		a, (m_back_buffer)
	ld 		b, a
	ld 		a, (m_front_buffer)
	ld 		(m_back_buffer), a
	ld 		a, b
	ld 		(m_front_buffer), a

	;; Desplazamos b a la derecha 2 bits
	srl 	b
	srl 	b
	ld 		l, b
	jp 	cpct_setVideoMemoryPage_asm


;;================================================================
;;	Llama a draw, move/saltar y clear y para dibujar mas de 1 entidad
;; 	Registros destruidos: AF, IX, BC 
;; 	Entrada HL -> puntero a función, A -> m_num_ball
;;================================================================
ent_doForAll_X:
	
	ld 		(metodoX), hl 				;; Cargamos en la direccion de metodo la función guardada en hl
	loop_doForAllX:
		push 	af 						;; Como a se destruye en bola_draw
		ld 		a, e_activada(ix)
		cp 		#TRUE					;; A-1 = 0, Z=1
		jr		nz,	no_activaX 			;; Si Z=1, salta a activada
		
		metodoX = . + 1 					;; Llamamos a la función cargada anteriormente con hl
		call 	ent_draw
		
		no_activaX:
		pop 	af 						;; Metemos af en la pila y lo sacamos tras bola_draw
		
		ld 		bc, #k_balls_size		;; Se introduce en bc el tamanyo que ocupa cada entidad bola 
		add		ix, bc 					;; Nos desplazamos en el vector hasta la siguiente bola

		dec  	a 						;; Mientras a indique que queden bolas para dibujar se repite el bucle
		jr 		nz, loop_doForAllX
	ret

;;================================================================
;;	Llama a draw, move/saltar y clear y para dibujar mas de 1 entidad
;; 	Registros destruidos: AF, IX, BC 
;; 	Entrada HL -> puntero a función, A -> m_num_ball
;;================================================================
ent_doForAll_Y:
	
	ld 		(metodoY), hl 				;; Cargamos en la direccion de metodo la función guardada en hl
	loop_doForAllY:
		push 	af 						;; Como a se destruye en bola_draw

		ld 		a, e_activada(iy)
		cp 		#TRUE					;; A-1 = 0, Z=1
		jr		nz,	no_activaY 			;; Si Z=1, salta a activada
		
		metodoY = . + 1 					;; Llamamos a la función cargada anteriormente con hl
		call 	ent_draw
		
		no_activaY:
		pop 	af 						;; Metemos af en la pila y lo sacamos tras bola_draw
		
		ld 		bc, #k_balls_size		;; Se introduce en bc el tamanyo que ocupa cada entidad bola 
		add		iy, bc 					;; Nos desplazamos en el vector hasta la siguiente bola

		dec  	a 						;; Mientras a indique que queden bolas para dibujar se repite el bucle
		jr 		nz, loop_doForAllY
	ret


;;======================================
;;	Dibujar una entidad
;;	Registros destruidos: AF, BC, DE, HL
;;  Entrada IX o IY -> Puntero a entidad
;;======================================
ent_draw:
	cp 		#TRUE
	jr 		z, uso_IX
		push iy
		pop ix

	uso_IX:
	;; Comprobamos si esta activa la entidad
	ld 		a, e_activada(ix)			;; e_activada es FALSE, 0
	cp 		#FALSE						;; 0-0 = 0, Z=1
	jr 		z, no_dibujar 				;; Si Z=1, salta a no_dibujar

	;; Comienzo de la memoria de video
	ld 		a, (m_back_buffer)
	ld 		d, a
	ld 		e, #0

	ld 		c, de_x(ix) 			;; C=Entity x, [0-79] bytes
	ld 		b, de_y(ix)				;; B=Entity y, [0-199] bytes
	call 	cpct_getScreenPtr_asm	;; Devuelve el puntero en HL

	;; Sprite propio
	ex 		de, hl 					;; DE = Puntero a memoria (HL -> Memory)
	ld 		l, de_sprite_l(ix)
	ld 		h, de_sprite_h(ix)
	ld 		b, de_h(ix)				;; Alto
	ld 		c, de_w(ix)				;; Ancho
	call 	cpct_drawSpriteMasked_asm

	no_dibujar:
ret


;;======================================
;;	Borrar una entidad
;;	Registros destruidos: A BC DE
;;  Entrada IX -> Puntero a entidad
;; 			C  -> pos X
;; 			B  -> pos Y
;;======================================
clear_general:
	;; Comienzo de la memoria de video
	ld 		a, (m_back_buffer)
	ld 		d, a
	ld 		e, #0

	call 	cpct_getScreenPtr_asm	;; Devuelve el puntero en HL

	;; Cubo negro
	ex 		de, hl 					;; DE = Puntero a memoria (HL -> Memory)
	ld 		a, #0					;; Color
	ld 		b, de_h(ix)				;; Alto
	ld 		c, de_w(ix)				;; Ancho
	call 	cpct_drawSolidBox_asm
ret

ent_clear:
	ld 		c, xAntigua(ix) 		;; C=Entity x, [0-79] bytes
	ld 		b, yAntigua(ix)			;; B=Entity y, [0-199] bytes
	call 	clear_general
ret

ent_clearAhoraIX:

	ld 		c, de_x(ix) 			;; C=Entity x, [0-79] bytes
	ld 		b, de_y(ix)				;; B=Entity y, [0-199] bytes
	call 	clear_general
ret


;;======================================
;;	Actualizar
;;	Registros destruidos: HL
;;======================================
ent_update:
	;; Salta a ent_movePersonaje, bola_update, bala_update
	ld 		h, e_upd_h(ix)
	ld 		l, e_upd_l(ix)
	jp		(hl)
ret

;;===============================================
;;  Para guardar las posiciones antiguas
;; 	Entrada IX -> Puntero a entidad
;;===============================================
intercambiarPosiciones:
	ld 		a, de_x(ix)
	ld 		xAntigua(ix), a
	ld 		a, de_y(ix)
	ld 		yAntigua(ix), a
ret

ent_restart:
	;; Borramos la antigua posicion
	call 	intercambiarPosiciones
	call 	ren_switchBuffers
	call 	ent_clear
ret


;;===============================================
;;  Para reiniciar la partida
;;===============================================
reiniciar_partida:
	call 	personaje_getPtrIX
	call 	reiniciar_personaje
		
	call 	bala_getPtrIX
	call 	reiniciar_bala

	call 	bola_grande_vector_getPtrIX
	call 	bola_erase

	call 	reiniciar_vidas
ret

bolas_pruebas:
	;; Creamos una bola en el array
	call	bola_new		;; vector + 1 y hl contiene la nueva bola
	ex 		de, hl
	ld 		hl, #bola_grande
	call 	bola_copy
	
	call	bola_new		;; vector + 1 y hl contiene la nueva bola
	ex 		de, hl
	ld 		hl, #bola_grande
	call 	bola_copy
ret 