;;-------------------------------------------------------------------------;;    This file is part of Catpang.
;;    Copyright (C) 2018-2019 Débora Galdeano González / Mª Inés Anton Castelló	
;;    Catpang is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    Catpang is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with Catpang.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------


;; include CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
.include "main.h.s"
.include "entity.h.s"
.include "nivel.h.s"

.area _DATA
;; Compresion y descompresion:
;; ===========================================================================
;; Uncompressed:     724 bytes			2D4 en hexadecimal
;; Compressed:       48 bytes
;; Space saved:      676 bytes

;; _level0_pack_size_z == 48
;; _level0_pack_size   == 724
;; _level0_pack_OFF_000 ==      0
;; _level0_pack_OFF_001 ==    500

decompress_buffer = #0x040										;; INICIO BUFFER de DESCOMPRESION
levelmaxsize = #0x2D4											;; TAM. NIVEL
decompress_buffer_end = decompress_buffer + levelmaxsize - 1    ;; FINAL BUFFER de DESCOMPRESION
																;; Despues empieza nuestro codigo
tileset_ptr = decompress_buffer + 500  ;;#_level0_pack_OFF_001 	;; 500 bytes
tilemap_ptr = decompress_buffer + 0    ;;#_level0_pack_OFF_000	;; 0 bytes

;; ===========================================================================

.area _CODE

;;======================================
;;	Descomprime un pack
;;  Destroy: AF, BC, DE, HL
;;  Entrada HL -> _XXXX_pack_end
;;======================================
descomprimir_pack:
	;;ld 		hl, #_XXXX_pack_end				;; Destino: Final del pack
	ld 		de, #decompress_buffer_end			;; Destino: Final del buffer
	call 	cpct_zx7b_decrunch_s_asm
ret


;;================================================
;;	Establecer valores para el dibujado del mapa
;;  Destroy: AF, BC, DE, HL, IX, IY
;;  Entrada DE -> _XXXX_W
;;================================================
set_Valores_tilemap:
	ld 		bc, #0x1914 				;; Viewport/view window 20x25 tiles
	;;ld 		de, #_XXXX_W			;; Tilemap width
	ld 		hl, #tileset_ptr			;; Tileset pointer
	call 	cpct_etm_setDrawTilemap4x8_ag_asm
ret


;;==================================================
;; Dibujar el tilemap en los 2 buffers de dibujado
;;==================================================
dibujar_tilemap:
	ld 		hl, #0xC000
	ld 		de, #tilemap_ptr
	call 	cpct_etm_drawTilemap4x8_ag_asm
	ld 		hl, #0x8000
	ld 		de, #tilemap_ptr
	call 	cpct_etm_drawTilemap4x8_ag_asm
ret


;;==================================================
;; Dibujar la intro del juego
;;==================================================
dibujar_menu:
	;; Esperamos a que se pulse SPACE para jugar o C para creditos
	loop_menu:
		call 	cpct_scanKeyboard_asm		;; Escanear todas las teclas
		ld 		hl, #Key_Space				;; HL = Key_Space Keycode
		call 	cpct_isKeyPressed_asm		;; Comprueba si Key_Space esta pulsada
		jr		z, space_no_pulsada
			jr 		jugar
		space_no_pulsada:
		call 	menu_draw
		call 	cpct_waitVSYNC_asm
		call 	ren_switchBuffers
		call 	menu_clear

	jr loop_menu
ret


;;======================================
;; Interrupt Service Routine
;;======================================
velInter = #6
unavariable: .db #velInter  ;;#6 velocidad normal
isr::
	;; Salvamos los registros
	ex 		af, af'
	exx
	push 	af
	push 	bc
	push 	de
	push 	hl
	push 	iy
	ld 		a, (unavariable)
	dec 	a
	ld 		(unavariable), a
	jr nz, volver
		call 	cpct_akp_musicPlay_asm
		ld 		a, #velInter  ;;#6 velocidad normal
		ld 		(unavariable), a
	volver:
	;; Restauramos los registros
	pop 	iy
	pop 	hl
	pop 	de
	pop 	bc
	pop 	af
	exx
	ex 		af, af'
	;; Colorines para ver las interrupciones
	;ld 		l, #16
	;ld 		a, (unavariable)
	;ld 		h, a
	;call cpct_setPALColour_asm
	;ld 		a, (unavariable)
	;inc 	a
	;cp 		#0x16
	;jr 		nz, continue
		;ld 		a, #0x10 ;; Reset colour
	;continue:
	;ld 		(unavariable), a
ret


;;======================================
;; Carga el mapa de nivel
;; Entrada, IX -> nivel
;;======================================
cargar_mapa_nivel:
	;; Salvamos los registros
	ex 		af, af'
	exx
	push 	af
	push 	bc
	push 	de
	push 	hl
	push 	iy
	push 	ix
	
	ld 		a, e_numNivel(ix)
	call 	cargar_mapa

	;; Restauramos los registros
	pop 	ix
	pop 	iy
	pop 	hl
	pop 	de
	pop 	bc
	pop 	af
	exx
	ex 		af, af'
ret



;;======================================
;;           Menu principal
;;======================================
_main::
	;; Desabilitamos el firmware
	call 	cpct_disableFirmware_asm

	;; Cambiamos la pila de sitio
	ld 		sp, #0x8000 					;; Escribe de 7FFF hacia abajo
	call 	ren_InitBuffer

	;; Valores de registro destruidos: AF, BC, HL
	ld 		c, #0						;; Modo 0 | Resolucion: 160x200 | 16 colores
	call 	cpct_setVideoMode_asm

	;; Asignamos la paleta
	ld 		hl, #_paleta_pal		;; Puntero a la paleta(array de colores) -> HL
	ld 		de, #16					;; Tamanyo: 16 colores
	call 	cpct_setPalette_asm

salir_menu:
	;; Llamada al dibujar menu
	ld 		a, #1
	ld 		e_numNivel(ix), a
	call  	dibujar_menu

;; Codigo para el juego
jugar:
	
	;; Inicializar musica
	ld 		de, #_musica
	call 	cpct_akp_musicInit_asm
	
	;; Set ISR
	ld 		hl, #isr
	call 	cpct_setInterruptHandler_asm

	reiniciar_nivel:
	call 	nivel_getPtrIX

	;; Dibujamos el mapa del juego
	call 	cargar_mapa_nivel

	ld 		a, e_numNivel(ix)
	call 	crear_niveles
	
	;;call 	bolas_pruebas
loop:
	
	;; HUD
	call 	hud_draw

	;; Metemos los datos del personaje en el registro IX y lo dibujamos
	call 	personaje_getPtrIX

	;; you_lose
	ld 		a, e_activada(ix)
	cp 		#FALSE
	jr 		z, comprobar

	call 	ent_update

	ld 		a, e_parpadeo(ix)
	cp 		#TRUE
	jr		z, no_dibujar_gato
	ld 		a, #TRUE 				;; Utiliza IX, por eso indica IX-TRUE
	call 	ent_draw

	no_dibujar_gato:

	;; Metemos los datos de la bala en el registro IX y comprobamos si hay que dibujarla
	ld 		a, de_x(ix)				;; Guardamos en A la pos. X de personaje
	ex 		af, af'					;; La pasamos a A'
	call 	bala_getPtrIX
	call 	ent_update
	ld 		a, #TRUE 				;; Utiliza IX, por eso indica IX-TRUE
	call 	ent_draw

	;; Metemos los datos de la bola grande en el registro IX y lo dibujamos
	ld 		ix, #bola_grande_vector ;; Cargamos la posición de inicio del vector de bolas
	call 	ent_update

	call 	cpct_waitVSYNC_asm
	call 	ren_newScene
	call 	nivel_getPtrIX
	ld 		a, e_nivelNumBalls(ix)
	cp 		#0
	jr 		z, comprobar_win
	jr 		loop

	you_lose:
	;call 		trans_lose
	ld 		a, #0

	espacio_temporal_reinicio:
	inc 	a
	halt
	cp 		#finLapso
	jr		nz, espacio_temporal_reinicio

	pulsar_para_reiniciar:
	
	call 	cpct_scanKeyboard_asm		;; Escanear todas las teclas
	call 	cpct_isAnyKeyPressed_asm		;; Comprueba si hay alguna tecla pulsada
	jr		z, pulsar_para_reiniciar	
	call 	reiniciar_partida
	jr 		reiniciar_nivel
	
	you_win:
	ld 		a, #0

	espacio_temporal_continuar:
	inc 	a
	halt
	cp 		#finLapso
	jr		nz, espacio_temporal_continuar

	pulsar_para_continuar:
	
	call 	cpct_scanKeyboard_asm		;; Escanear todas las teclas
	call 	cpct_isAnyKeyPressed_asm		;; Comprueba si hay alguna tecla pulsada

	jr		z, pulsar_para_continuar

	call 	reiniciar_partida

	call 	nivel_getPtrIX
	ld 		a, e_numNivel(ix)
	cp 		#6
	jp 		z, salir_menu
	inc  	a
	ld 		e_numNivel(ix), a
	jp 		reiniciar_nivel

	;; Bucle
	jr 		loop

	comprobar:
	call 	trans_lose
	jp 		you_lose

	comprobar_win:
	call 	nivel_getPtrIX
	ld 		a, e_numNivel(ix)
	cp 		#2
	jp 		z, egg
	call 	trans_win
	jp 		you_win
	egg:
	call 	easter_egg
	jp 		you_win